#define SDC_FLOATING_BUS    0xFF
#define SDC_BAD_RESPONSE    SDC_FLOATING_BUS
#define SDC_ILLEGAL_CMD     0x04
#define SDC_GOOD_CMD        0x00

#define BLOCKLEN_64                 0x0040
#define BLOCKLEN_128                0x0080
#define BLOCKLEN_256                0x0100
#define BLOCKLEN_512                0x0200
//definicja typow potwierdzen
#define R1 0x00
#define R1b 0x01
#define R2 0x02
#define R3 0x03
#define NODATA 0x00
#define MOREDATA 0x01

//definicje znacznikow
/* Data Token */
#define DATA_START_TOKEN            0xFE
#define DATA_MULT_WRT_START_TOK     0xFC
#define DATA_MULT_WRT_STOP_TOK      0xFD
//definicje kodow potwierdzen
/* Data Response */
#define DATA_ACCEPTED               0x05 //0b00000101
#define DATA_CRC_ERR                0x0b //0b00001011
#define DATA_WRT_ERR                0x0d //0b00001101
// Bit 7 = 0, Bit 6 = 1, Bits 5-0=Command Code
#define     GO_IDLE_STATE        0x40		//0 is the 6-bit command
#define     SEND_OP_COND         0x41		//1        
#define     SEND_CSD             0x49		//9
#define     SEND_CID             0x4a		//10
#define     STOP_TRANSMISSION    0x4c		//12
#define     SEND_STATUS          0x4d		//13
#define     SET_BLOCKLEN         0x50		//16
#define     READ_SINGLE_BLOCK    0x51		//17
#define     READ_MULTI_BLOCK     0x52		//18
#define     WRITE_SINGLE_BLOCK   0x58		//24    
#define     WRITE_MULTI_BLOCK    0x59		//25
#define     TAG_SECTOR_START     0x60		//32
#define     TAG_SECTOR_END       0x61		//33
#define     UNTAG_SECTOR         0x62		//34
#define     TAG_ERASE_GRP_START  0x63		//35 
#define     TAG_ERASE_GRP_END    0x64		//36
#define     UNTAG_ERASE_GRP      0x65		//37
#define     ERASE                0x66		//38
#define     SD_APP_OP_COND       0x69		//41
#define     LOCK_UNLOCK          0x71		//49
#define     APP_CMD              0x77		//55
#define     READ_OCR             0x7a		//58
#define     CRC_ON_OFF           0x7b		//59

/*definicja symbolicznych nazw komend uywanych w wywolaniu procedury wyslania komendy */
enum
{
    cmdGO_IDLE_STATE,
    cmdSEND_OP_COND,  
    cmdSEND_CSD,         
    cmdSEND_CID,           
    cmdSTOP_TRANSMISSION,  
    cmdSEND_STATUS,        
    cmdSET_BLOCKLEN,       
    cmdREAD_SINGLE_BLOCK,  
    cmdREAD_MULTI_BLOCK,   
    cmdWRITE_SINGLE_BLOCK, 
    cmdWRITE_MULTI_BLOCK,  
    cmdTAG_SECTOR_START,   
    cmdTAG_SECTOR_END,     
    cmdUNTAG_SECTOR,       
    cmdTAG_ERASE_GRP_START, 
    cmdTAG_ERASE_GRP_END,  
    cmdUNTAG_ERASE_GRP,    
    cmdERASE,              
    cmdLOCK_UNLOCK,        
    cmdSD_APP_OP_COND,   
    cmdAPP_CMD,            
    cmdREAD_OCR,           
    cmdCRC_ON_OFF,
	cmdSD        
};

/*----------------------------------------------------------------------------------------------------
 definicja tablicy zawieracej kod komendy, sume kontrolna, typ potwierdzenia
 oraz znacznik kontynuacji odbierania danych
-----------------------------------------------------------------------------------------------------*/
const unsigned char CmdTab[23][4] =
{
    //cmd                crc   response
    {GO_IDLE_STATE,      0x95, R1, NODATA},
    {SEND_OP_COND,       0xF9, R1, NODATA},
    {SEND_CSD,           0xAF, R1, MOREDATA},
    {SEND_CID,           0x1B, R1, MOREDATA},
    {STOP_TRANSMISSION,  0xC3, R1, NODATA},
    {SEND_STATUS,        0xAF, R2, NODATA},
    {SET_BLOCKLEN,       0xFF, R1, NODATA},
    {READ_SINGLE_BLOCK,  0xFF, R1, MOREDATA},
    {READ_MULTI_BLOCK,   0xFF, R1, MOREDATA},
    {WRITE_SINGLE_BLOCK, 0xFF, R1, MOREDATA},
    {WRITE_MULTI_BLOCK,  0xFF, R1, MOREDATA}, 
    {TAG_SECTOR_START,   0xFF, R1, NODATA},
    {TAG_SECTOR_END,     0xFF, R1, NODATA},
    {UNTAG_SECTOR,       0xFF, R1, NODATA},
    {TAG_ERASE_GRP_START, 0xFF, R1, NODATA},
    {TAG_ERASE_GRP_END,  0xFF, R1, NODATA},
    {UNTAG_ERASE_GRP,    0xFF, R1, NODATA},
    {ERASE,              0xDF, R1b, NODATA},
    {LOCK_UNLOCK,        0x89, R1b, NODATA},  
    {SD_APP_OP_COND,     0xE5, R1, NODATA},
    {APP_CMD,            0x73, R1, NODATA},
    {READ_OCR,           0x25, R3, NODATA},
    {CRC_ON_OFF,         0x25, R1, NODATA}
};
//definicje kodow bledow
typedef enum 
{
    sdcValid=0,                 // wszsytko poprawnie
    sdcCardInitCommFailure,     // nie ustanowioni polaczenia z karta
    sdcCardNotInitFailure,      // karta nie przeszla fazy incjalizacji
    sdcCardInitTimeout,         // timeout inicjalizacji karty
    sdcCardTypeInvalid,         // nie mozna zdefiniowac karty 
    sdcCardBadCmd,              // karta nie rozpoznala komendy
    sdcCardTimeout,             // timeout w czasie sekwencji zapisu odczyty lub kasowania
    sdcCardCRCError,            // wystapil blad CRC w czasie odczytu, dane powinny byc odrzucone
    sdcCardDataRejected,        // blad CRC danych przesylanych
    sdcEraseTimedOut            // timeout kasowania danych 
}SDC_Error;


